/*
 * Created on 23. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.ntnu.item.jt.system.configbeans.AVEngineBean;
import edu.ntnu.item.jt.system.configbeans.AVRealTimeScanBean;
import edu.ntnu.item.jt.system.configbeans.AVUpdateInfoBean;
import edu.ntnu.item.jt.system.configbeans.FilterBean;

/**
 * This class represents the "Edit AVE" view in the config editor. 
 * The class holds the associated AVEngineBean, and the text fields.
 * @author Thomas Langerud
 *
 */
public class AVEnginePanel extends JPanel implements ActionListener{

	private AVEngineBean avEngineBean;
	private JTextField name, execPath, parameters, remoteLogPath, updateExecPath, updateParameter,updateLogPath, updateSuccessIndicator, rtLogPath, rtResult, filterResultIdentifier,resultSuffix,resultPrefix;
	private GUIConfigEditor editor;
	private JButton saveChangesButton;
	
	
	public AVEnginePanel(AVEngineBean avEngine, GUIConfigEditor editor) {
		
		this.editor = editor;
		avEngineBean = avEngine;
	}
	
	/**
	 * Method to create and populate the text fields that belongs to this panel
	 */
	public void draw() {
		
		this.setLayout(new GridLayout(14,2));
		
		this.add(new JLabel("Name"));
		this.name = new JTextField(avEngineBean.getName());
		this.add(name);
		
		this.add(new JLabel("Executable path"));
		execPath = new JTextField(avEngineBean.getExecutablePath());
		this.add(execPath);
		
		this.add(new JLabel("Parameters"));
		parameters = new JTextField(avEngineBean.getParameters());
		this.add(parameters);
		
		this.add(new JLabel("Remote log path"));
		remoteLogPath = new JTextField(avEngineBean.getRemoteLogFilePath());
		this.add(remoteLogPath);
		
		updateExecPath = new JTextField();
		updateParameter = new JTextField();
		updateLogPath = new JTextField();
		updateSuccessIndicator = new JTextField();
		
		
		if(avEngineBean.getAvUpdate() != null) {
			updateExecPath.setText(avEngineBean.getAvUpdate().getUpdateExecutable());
			updateParameter.setText(avEngineBean.getAvUpdate().getUpdateParameters());
			updateLogPath.setText(avEngineBean.getAvUpdate().getUpdateLogPath());
			updateSuccessIndicator.setText(avEngineBean.getAvUpdate().getUpdateSuccessIndicator());
		}
		
		this.add(new JLabel("Update executable path"));
		this.add(updateExecPath);
		
		this.add(new JLabel("Update parameters"));
		this.add(updateParameter);
		
		this.add(new JLabel("Update log file"));
		this.add(updateLogPath);
		
		this.add(new JLabel("Sucess indicator"));
		this.add(updateSuccessIndicator);
		
		rtLogPath = new JTextField();
		rtResult = new JTextField();
		
		if(avEngineBean.getRealtimeScanBean() != null) {
			rtLogPath.setText(avEngineBean.getRealtimeScanBean().getLogFilePath());
			rtResult.setText(avEngineBean.getRealtimeScanBean().getResultIdentifier());
		}
		
		this.add(new JLabel("Real-time log"));
		this.add(rtLogPath);
		this.add(new JLabel("Real-time success indicator"));
		this.add(rtResult);
		
		filterResultIdentifier = new JTextField();
		resultPrefix = new JTextField();
		resultSuffix = new JTextField();
		
		if(avEngineBean.getFilter() != null) {
			filterResultIdentifier.setText(avEngineBean.getFilter().getResultIdentifier());
			resultPrefix.setText(avEngineBean.getFilter().getResultPrefix());
			resultSuffix.setText(avEngineBean.getFilter().getResultSuffix());
		}
		
		this.add(new JLabel("Log filter identifier"));
		this.add(filterResultIdentifier);
		this.add(new JLabel("Result prefix"));
		this.add(resultPrefix);
		this.add(new JLabel("Result suffix"));
		this.add(resultSuffix);
		
		saveChangesButton = new JButton("Save changes");
		saveChangesButton.addActionListener(this);
		this.add(saveChangesButton);
	
	}
	
	/**
	 * This method updates the panel with the data values associated with the 
	 * bean held by this class
	 */
	public void refresh() {
		name.setText(avEngineBean.getName()); 
		execPath.setText(avEngineBean.getExecutablePath()); 
		parameters.setText(avEngineBean.getParameters());  
		remoteLogPath.setText(avEngineBean.getRemoteLogFilePath());
		
		if(avEngineBean.getAvUpdate() != null) {
			updateExecPath.setText(avEngineBean.getAvUpdate().getUpdateExecutable());
			updateParameter.setText(avEngineBean.getAvUpdate().getUpdateParameters());
			updateLogPath.setText(avEngineBean.getAvUpdate().getUpdateLogPath());
			updateSuccessIndicator.setText(avEngineBean.getAvUpdate().getUpdateSuccessIndicator());
		}
		if(avEngineBean.getRealtimeScanBean() != null) {
			rtLogPath.setText(avEngineBean.getRealtimeScanBean().getLogFilePath());
			rtResult.setText(avEngineBean.getRealtimeScanBean().getResultIdentifier());
		}
		if(avEngineBean.getFilter() != null) {
			filterResultIdentifier.setText(avEngineBean.getFilter().getResultIdentifier());
			resultPrefix.setText(avEngineBean.getFilter().getResultPrefix());
			resultSuffix.setText(avEngineBean.getFilter().getResultSuffix());
		}
	}
	
	/**
	 * This method fetches the text from the panel text fields and updates the associated bean
	 * with the data
	 */
	public void save() {
		avEngineBean.setName(name.getText());
		avEngineBean.setExecutablePath(execPath.getText());
		avEngineBean.setParameters(parameters.getText());
		avEngineBean.setRemoteLogFilePath(remoteLogPath.getText());
		
		if( (updateExecPath.getText() != "" ) || (updateParameter.getText() != "") || (updateLogPath.getText()!= "" ) || (updateSuccessIndicator.getText() != "")){
			
			if(avEngineBean.getAvUpdate() == null) {
				avEngineBean.setAvUpdate(new AVUpdateInfoBean()) ;
			}
			avEngineBean.getAvUpdate().setUpdateExecutable(updateExecPath.getText());
			avEngineBean.getAvUpdate().setUpdateParameters(updateParameter.getText());
			avEngineBean.getAvUpdate().setUpdateLogPath(updateLogPath.getText());
			avEngineBean.getAvUpdate().setUpdateSuccessIndicator(updateSuccessIndicator.getText());
		}
		if( (rtLogPath.getText() != "" ) || (rtResult.getText() != "") ){
		
			if(avEngineBean.getRealtimeScanBean() == null) {
				avEngineBean.setRealtimeScanBean(new AVRealTimeScanBean());
			}
			avEngineBean.getRealtimeScanBean().setLogFilePath(rtLogPath.getText());
			avEngineBean.getRealtimeScanBean().setResultIdentifier(rtResult.getText());
		}
		if( (filterResultIdentifier.getText() != "" ) || (resultPrefix.getText() != "") || (resultSuffix.getText()!= "" )){

			if(avEngineBean.getFilter() == null) {
				avEngineBean.setFilter(new FilterBean());
			}
			avEngineBean.getFilter().setResultIdentifier(filterResultIdentifier.getText());
			avEngineBean.getFilter().setResultPrefix(resultPrefix.getText());
			avEngineBean.getFilter().setResultSuffix(resultSuffix.getText());
		}
		
	}

	public AVEngineBean getAvEngineBean() {
		return avEngineBean;
	}

	public void setAvEngineBean(AVEngineBean avEngineBean) {
		this.avEngineBean = avEngineBean;
	}


	@Override
	public void actionPerformed(ActionEvent e) {

		if(e.getSource().equals(saveChangesButton)) {
			save();
			refresh();
		}
		
	}
	
}
