/*
 * Created on 23. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import edu.ntnu.item.jt.system.configbeans.DynamicAnalysisToolBean;
import edu.ntnu.item.jt.system.configbeans.VMBean;

/**
 * This class is a representation of the tabbed pane that contains the tool panels
 * in the "Edit tool" view of the config editor.
 * @author Thomas Langerud
 *
 */
public class DynamicToolPanel extends JTabbedPane implements ActionListener{

	private GUIConfigEditor editor;
	public JTextField name, path, parameters, resultPath;
	private ArrayList<ToolPanel> toolPanels;
	private ArrayList<DynamicAnalysisToolBean> toolBeans;
	
	public DynamicToolPanel(VMBean vmBean, GUIConfigEditor editor) {
		super();
		this.editor = editor;
		toolBeans = new ArrayList<DynamicAnalysisToolBean>();
		toolBeans = vmBean.getAnalysisTools();
		
		toolPanels = new ArrayList<ToolPanel>();
		
		for(DynamicAnalysisToolBean toolBean : toolBeans) {
			ToolPanel tool = new ToolPanel(toolBean, editor, this);
			tool.draw();
			toolPanels.add(tool);
		}
		
	}
	
	/**
	 * Method to create and populate the text fields on the panels that belongs to 
	 * this tabbed panel
	 */
	public void draw() {

		int toolCounter = 1;
		for(ToolPanel tool : toolPanels) {
			
			this.addTab("Tool "+toolCounter, tool);
			toolCounter++;
		}
		
	}
	
	/**
	 * Method to update the text fields on the tool panels belonging to this tabbed pane.
	 */
	public void refresh() {
		for(ToolPanel toolPanel : toolPanels) {
			toolPanel.refresh();

		}
	}
	
	/**
	 * Method to save the text from the text fields on the associated panels and update the
	 * beans held by the panels. 
	 */
	public void save() {
		for(ToolPanel toolPanel : toolPanels) {
			toolPanel.save();
		}
		
	}
	
	/**
	 * This method adds a new tool panel to the tabbed pane
	 */
	private void addTool() {
		
		DynamicAnalysisToolBean newToolBean = new DynamicAnalysisToolBean();
		newToolBean.setToolExecutablePath("executable path");
		newToolBean.setToolParameters("parameters");
		newToolBean.setToolResultFile("result file");
		newToolBean.setExecuteMalwareExplicitly(false);
		newToolBean.setName("New tool");
		
		ToolPanel newToolPanel = new ToolPanel(newToolBean, editor, this);
		newToolPanel.draw();
		toolBeans.add(newToolBean);
		toolPanels.add(newToolPanel);
		this.addTab("New tool", newToolPanel);
		refresh();
		
	}
	public ArrayList<DynamicAnalysisToolBean> getTools() {
		return toolBeans;
	}

	public void setTools(ArrayList<DynamicAnalysisToolBean> tools) {
		this.toolBeans = tools;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("add")) {
			addTool();
		}
		
	}

	public ArrayList<ToolPanel> getToolPanels() {
		return toolPanels;
	}

	public void setToolPanels(ArrayList<ToolPanel> toolPanels) {
		this.toolPanels = toolPanels;
	}

	public ArrayList<DynamicAnalysisToolBean> getToolBeans() {
		return toolBeans;
	}

	public void setToolBeans(ArrayList<DynamicAnalysisToolBean> toolBeans) {
		this.toolBeans = toolBeans;
	}
	
	/**
	 * This method deletes the selected panels from the tabbed panel, and also removed the
	 * associated bean. 
	 */
	public void deleteTool() {
		
		if(toolPanels.size() > 1 ) {
			toolPanels.remove(this.getSelectedIndex());
			toolBeans.remove(this.getSelectedIndex());
		}
		refresh();
	}
}
