/*
 * Created on 2. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

import edu.ntnu.item.jt.configloader.PropertiesReader;
import edu.ntnu.item.jt.system.*;
import edu.ntnu.item.jt.system.errors.*;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.prefs.*;

/**
 * 
 * This class is responsible for creating and managing the GUI.
 * 
 * @author Thomas Langerud
 */
public class GUI implements ActionListener, KeyListener, WindowListener, Runnable{

		JFrame mainWindow ;
		JPanel mainPanel, upperPanel, middlePanel, bottomPanel; 
		JPanel upperLeft, upperCenter, upperRight; // To improve the layout of buttons and fields
		JLabel samplePath, configLabel, propsLabel; 
		JTextField samplePathField, configField, propsField ;
		JTextArea consoleTextArea;
		JButton scan, clear, execute, analyze, update, open1, open2, open3, clear1, clear2, clear3 ;
		JMenu menuSystem, menuHelp, menuOutput, menuEdit;
		JMenuBar menuBar;
		JMenuItem menuItemExit, menuItemStart, menuItemAbout, menuItemHelp, menuItemSave, menuItemConfEditor;
		JScrollPane jScrollPane1;
		JFileChooser fc, saveFC;
		PowerScan ps;
		
		final int FIELD_MALWARE = 1;
		final int FIELD_CONFIG = 2;
		final int FIELD_PROPS = 3;
		
		final int OPERATION_SCAN = 1;
		final int OPERATION_EXECUTE = 2;
		final int OPERATION_ANALYZE = 3;
		final int OPERATION_INITIALIZE = 4;
		final int OPERATION_UPDATE = 5;
		
		int operation = 1;
		int carretPosition = 0;
		
		// Preferences from the windows registry
		Preferences prefs;
		
		final String REGPATH_CONFIG = "edu.ntnu.item.jt.gui.config";
		final String REGPATH_PROPS = "edu.ntnu.item.jt.gui.prefs";
		final String REGPATH_OUTPUTSAVE = "edu.ntnu.item.jt.gui.savepath";
		final String CLOSE_WARNING = "Please close this application without using System->Exit";
		final String CLOSE_CONSOLE_WARNING = "\n\n\nNOTE:\n\n->-> Please make sure not to terminate this application before operations have completed, as termination may result in unpredictable state!";
		final String EXIT_WARNING_QUESTION = "Are you sure you would like to exit";
		final String EMPTY_SAMPLE_PATH_ERROR = "!! WARNING !!  --> Malware sample path may not be empty";
		final String HELP_STRING = "Usage:\n\n* Enter path to config file and properties file into the appropriate fields, or use the open file dialog.\n\n* Choose 'System' -> 'Start'\n\n* Enter path to sample file into appropriate field or use open dialog\n\n* Choose analysis action with button";
		final String ABOUT_STRING = "PowerScan\n\nConsole for PowerScan Malware Analysis Framework\n\nBy:\n\nJran Vagnby Lillesand\nThomas Langerud";
		final String MAIN_WINDOW_TITLE = "PowerScan";
		final String LOOK_AND_FEEL_PATH = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		final String EMPTY_PROPS_PATH_ERROR = "A properties file must be given";
		
		/**
		 * Public constructor.
		 * This method sets up all graphics elements, and places them on their respective panels.
		 * It also assigns action listeners and action commands to the various elements.
		 * Finally, it creates the menu line and the items associated with it.
		 */
		public GUI(){
			
			mainWindow = new JFrame(MAIN_WINDOW_TITLE);
			mainWindow.setLocation(100,200);
			mainWindow.setSize(500,400);

			mainWindow.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
					    
			try {
		        UIManager.setLookAndFeel(LOOK_AND_FEEL_PATH);
		    } 
			catch (InstantiationException e) {
				mainWindow.setDefaultLookAndFeelDecorated(true);
		    } 
			catch (ClassNotFoundException e) {
				mainWindow.setDefaultLookAndFeelDecorated(true);
		    } 
			catch (UnsupportedLookAndFeelException e) {
				mainWindow.setDefaultLookAndFeelDecorated(true);
		    } 
			catch (IllegalAccessException e) {
				mainWindow.setDefaultLookAndFeelDecorated(true);
		    }
			
		    mainWindow.setResizable(false);
			mainWindow.addWindowListener(this);
			
			mainPanel = new JPanel();
			mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS));
			
			open1 = new JButton("Open");
			open2 = new JButton("Open");
			open3 = new JButton("Open");
			
			// Need to be disabled until constructor completes, or the file chooser dialog will throw a nullPointerException
			open1.setEnabled(false);
			open2.setEnabled(false);
			open3.setEnabled(false);
			clear1 = new JButton("Clear");
			clear2 = new JButton("Clear");
			clear3 = new JButton("Clear");
			clear1.setEnabled(false);
			clear2.setEnabled(false);
			clear3.setEnabled(false);
			
			open1.addActionListener(this);
			open2.addActionListener(this);
			open3.addActionListener(this);
			open1.setActionCommand(""+FIELD_MALWARE);
			open2.setActionCommand(""+FIELD_CONFIG);
			open3.setActionCommand(""+FIELD_PROPS);
			clear1.addActionListener(this) ;
			clear2.addActionListener(this);
			clear3.addActionListener(this);
			clear1.setActionCommand(""+FIELD_MALWARE);
			clear2.setActionCommand(""+FIELD_CONFIG);
			clear3.setActionCommand(""+FIELD_PROPS);
			
			upperPanel = new JPanel();
			// Lay out text, buttons and input fields on separate panels to get desired layout
			upperLeft = new JPanel();
			upperCenter = new JPanel();
			upperRight = new JPanel();
			
			upperLeft.setLayout(new GridLayout(3,1));
			upperCenter.setLayout(new GridLayout(3,1));
			upperRight.setLayout(new GridLayout(3,1));
			
			upperPanel.setLayout(new GridLayout(1,3));
			
			upperLeft.add(samplePath = new JLabel("  Malware sample path:"));
			upperCenter.add(open1);
			upperCenter.add(clear1);
			upperRight.add(samplePathField = new JTextField("",15));

			upperLeft.add(configLabel = new JLabel("  Config file path:"));
			upperCenter.add(open2);
			upperCenter.add(clear2);
			upperRight.add(configField = new JTextField("",15));

			upperLeft.add(propsLabel = new JLabel("  Properties file path:"));
			upperCenter.add(open3);
			upperCenter.add(clear3);
			upperRight.add(propsField = new JTextField("",15));			
			
			upperPanel.add(upperLeft);
			upperPanel.add(upperCenter);
			upperPanel.add(upperRight);
			
			middlePanel = new JPanel();
			middlePanel.add(clear = new JButton("Clear"));
			middlePanel.add(update = new JButton("Update"));
			middlePanel.add(scan = new JButton("Scan"));
			middlePanel.add(execute = new JButton("Execute"));
			middlePanel.add(analyze = new JButton("Dyn. Analyze"));
			scan.setEnabled(false);
			update.setEnabled(false);
			execute.setEnabled(false);
			analyze.setEnabled(false);
			
			scan.addActionListener(this);
			clear.addActionListener(this);
			update.addActionListener(this);
			execute.addActionListener(this);
			analyze.addActionListener(this);
			// If we wish to responde to ENTER and ESC key events, we need a KeyListener
			samplePathField.addKeyListener(this);

			consoleTextArea = new JTextArea();
			// Add 'console' color scheme ;)
			consoleTextArea.setBackground(Color.BLACK);
			consoleTextArea.setForeground(Color.green);
			consoleTextArea.setText("Hello\nWelcome to 'PowerScan' console\nPlease wait\n");
			consoleTextArea.setColumns(50);
			consoleTextArea.setRows(10);
			consoleTextArea.setLineWrap(true);
			consoleTextArea.setWrapStyleWord(true);
	        consoleTextArea.setEditable(false);
	        // The console area should be scrollable to allow more text
	        jScrollPane1 = new JScrollPane(consoleTextArea);
	        jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	        
			bottomPanel = new JPanel();
			bottomPanel.add(jScrollPane1);
			
			mainPanel.add(upperPanel);
			mainPanel.add(middlePanel);
			mainPanel.add(bottomPanel);
			
			
			// Create the menu items
			menuSystem = new JMenu("System");
			menuSystem.setMnemonic(KeyEvent.VK_F);
			
			menuHelp = new JMenu("Help");
			menuHelp.setMnemonic(KeyEvent.VK_H);
			
			menuOutput = new JMenu("Output");
			menuOutput.setMnemonic(KeyEvent.VK_O);
			
			menuEdit = new JMenu("Edit");
			menuEdit.setMnemonic(KeyEvent.VK_E);
			
			menuItemExit = new JMenuItem("Exit", KeyEvent.VK_X);
			menuItemStart = new JMenuItem("Start", KeyEvent.VK_S);
			menuItemAbout = new JMenuItem("About", KeyEvent.VK_A);
			menuItemHelp = new JMenuItem("Help", KeyEvent.VK_H);
			menuItemSave = new JMenuItem("Save", KeyEvent.VK_V);
			menuItemConfEditor = new JMenuItem("Config editor", KeyEvent.VK_E);
			
			//The menu items also require an actionListener
			menuItemExit.addActionListener(this);
			menuItemStart.addActionListener(this);
			menuItemAbout.addActionListener(this);
			menuItemHelp.addActionListener(this);
			menuItemSave.addActionListener(this);
			menuItemConfEditor.addActionListener(this);
			
			//Add the menu items to their respective menus
			menuSystem.add(menuItemStart);
			menuSystem.add(menuItemExit);
			
			menuHelp.add(menuItemHelp);
			menuHelp.add(menuItemAbout);
			
			menuOutput.add(menuItemSave);
			
			menuEdit.add(menuItemConfEditor);
			
			//Add the menus to the menu bar
			menuBar = new JMenuBar();
			menuBar.add(menuSystem);
			menuBar.add(menuEdit);
			menuBar.add(menuOutput);
			menuBar.add(menuHelp);
			
			//Add the menu bar to the frame
			mainWindow.setJMenuBar(menuBar);
			
			mainWindow.getContentPane().add(mainPanel);
			mainWindow.setVisible(true);
			
			// Get a reference to the HKCU\Software\Javasoft\Prefs hive
			prefs = Preferences.userRoot();
			configField.setText(prefs.get(REGPATH_CONFIG, "config"+System.getProperty("file.separator")+"config.xml"));
			propsField.setText(prefs.get(REGPATH_PROPS, "config"+System.getProperty("file.separator")+"powerscan.properties"));
			System.getProperty("file.separator");
			// Once the JFileChooser object has been created, it is possible to choose files to open
			fc = new JFileChooser(prefs.get(REGPATH_CONFIG, "c:\\"));
			
			// Do this at the end of the constructor
			open1.setEnabled(true);
			open2.setEnabled(true);
			open3.setEnabled(true);
			setInInitialState();
			appendInfo("Now ready");
			appendInfo("Usage: input path to config and properties files, select System->Start");
		}
		
		/**
		 * This method needs to be called before invoking any operation, as this method
		 * is responsible for creating a new PowerScan-object from the specified config file
		 */
		private void initializePowerScan() {
			
			consoleTextArea.setText("");
			
			// write paths to the registry
			if( !configField.getText().equals("")) {
				prefs.put(REGPATH_CONFIG, configField.getText());
			}
			if( !propsField.getText().equals("")) {
				prefs.put(REGPATH_PROPS, propsField.getText());
			}
			// Start this as a separate thread to avoid blocking the GUI
			operation = OPERATION_INITIALIZE;
			Thread t = new Thread(this);
			t.start();
		}
		
		
		/**
		 * Method that disables the operation buttons and the 'exit' menu choice
		 */
		private void disableOperations(){
			// All buttons should change at once
			synchronized(this){
				scan.setEnabled(false);
				execute.setEnabled(false);
				analyze.setEnabled(false);
				update.setEnabled(false);
				
				menuItemExit.setEnabled(false);
				menuItemStart.setEnabled(false);
				
				menuSystem.setEnabled(false); 
				menuHelp.setEnabled(false); 
				menuOutput.setEnabled(false); 
				menuEdit.setEnabled(false);
				
				samplePathField.setEnabled(false);
				open1.setEnabled(false);
				clear1.setEnabled(false);
				configField.setEnabled(false);
				open2.setEnabled(false);
				clear2.setEnabled(false);
				propsField.setEnabled(false);
				open3.setEnabled(false);
				clear3.setEnabled(false);
			}
		}
		
		/**
		 * Sets the input fields and buttons disabled, and informs the user via the console text area and 
		 * dialog box about the fact that the system needs to be restarted after an operation. 
		 */
		private void setInDoneState() {
			synchronized(this){
				scan.setEnabled(false);
				execute.setEnabled(false);
				analyze.setEnabled(false);
				update.setEnabled(false);
				
				samplePathField.setEnabled(false);
				open1.setEnabled(false);
				clear1.setEnabled(false);
				configField.setEnabled(false);
				open2.setEnabled(false);
				clear2.setEnabled(false);
				propsField.setEnabled(false);
				open3.setEnabled(false);
				clear3.setEnabled(false);
				
				menuSystem.setEnabled(true); 
				menuHelp.setEnabled(true); 
				menuOutput.setEnabled(true); 
				menuEdit.setEnabled(true);
				
				menuItemExit.setEnabled(true);
				menuItemStart.setEnabled(false);
				
				appendInfo("Due to restrictions in the Vix API, the PowerScan systems needs to be restarted before next operation");
				showDialogWithWidthRestriction("Due to restrictions in the Vix API, the PowerScan systems needs to be restarted before next operation", "Restart required!", 40, JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		private void setInAfterInitializationState() {
			synchronized(this){
				scan.setEnabled(true);
				execute.setEnabled(true);
				analyze.setEnabled(true);
				update.setEnabled(true);
				
				samplePathField.setEnabled(true);
				open1.setEnabled(true);
				clear1.setEnabled(true);
				configField.setEnabled(false);
				open2.setEnabled(false);
				clear2.setEnabled(false);
				propsField.setEnabled(false);
				open3.setEnabled(false);
				clear3.setEnabled(false);
				
				menuSystem.setEnabled(true); 
				menuHelp.setEnabled(true); 
				menuOutput.setEnabled(true); 
				menuEdit.setEnabled(true);
				
				menuItemExit.setEnabled(true);
				menuItemStart.setEnabled(true);

			}
		}
		
		private void setInInitialState() {
			synchronized(this){
				scan.setEnabled(false);
				execute.setEnabled(false);
				analyze.setEnabled(false);
				update.setEnabled(false);
				
				samplePathField.setEnabled(true);
				open1.setEnabled(true);
				clear1.setEnabled(true);
				configField.setEnabled(true);
				open2.setEnabled(true);
				clear2.setEnabled(true);
				propsField.setEnabled(true);
				open3.setEnabled(true);
				clear3.setEnabled(true);
				
				menuSystem.setEnabled(true); 
				menuHelp.setEnabled(true); 
				menuOutput.setEnabled(true); 
				menuEdit.setEnabled(true);
				
				menuItemExit.setEnabled(true);
				menuItemStart.setEnabled(true);

			}
		}
		
		/**
		 * Method that clears the content on the console text area
		 */
		private void clearMethod(){
			consoleTextArea.setText("");	
		}
		
		/**
		 * Method that invokes the scan command in PowerScan.
		 * Starts a new thread, to avoid blocking the GUI
		 */
		private void scanMethod(){

			operation = OPERATION_SCAN;
			Thread t = new Thread(this);
			t.start();
		}

		/**
		 * Method that invokes the execute malware command in PowerScan.
		 * Starts a new thread, to avoid blocking the GUI
		 */
		private void executeMethod() {

			operation = OPERATION_EXECUTE;
			Thread t = new Thread(this);
			t.start();
		}
		
		/**
		 * Method that invokes the analysis tools command in PowerScan.
		 * Starts a new thread, to avoid blocking the GUI
		 */
		private void analyzeMethod() {
			
			operation = OPERATION_ANALYZE;
			Thread t = new Thread(this);
			t.start();
		}
		
		/**
		 * Method that invokes the update command in PowerScan
		 * Starts a new thread, to avoid blocking the GUI
		 */
		private void updateMethod(){
			operation = OPERATION_UPDATE;
			Thread t = new Thread(this);
			t.start();
		}
		
		/**
		 * Method to append text to the console text area
		 * @param info The text to be appended to the console
		 */
		public void appendInfo(String info) {
			consoleTextArea.append(info + "\n\n");

			// trick to scroll down to the bottom of the text area
			consoleTextArea.selectAll();
			carretPosition = consoleTextArea.getSelectionEnd();
			consoleTextArea.select(carretPosition,carretPosition);
		}
		
		/**
		 * Method to append text to the console text area, scroll down and 
		 * display a warning dialog box with the text.
		 * Note: The text shown in the dialog box is not line wrapped, so the dialog box
		 * takes the size og the text horisontally
		 * @param text The text to be appended to the console and shown in the dialog box
		 */
		public void appendWarning(String text){
		
			consoleTextArea.append("Warning: " +text + "\n\n");

			// trick to scroll down to the bottom of the text area
			consoleTextArea.selectAll();
			carretPosition = consoleTextArea.getSelectionEnd();
			consoleTextArea.select(carretPosition,carretPosition);

		}
		
		/**
		 * Method to append text to the console text area, scroll down and 
		 * display a error dialog box with the text
		 * @param text
		 */
		public void appendError(String text){

			consoleTextArea.append("Error:" + text + "\n\n");

			// trick to scroll down to the bottom of the text area
			consoleTextArea.selectAll();
			carretPosition = consoleTextArea.getSelectionEnd();
			consoleTextArea.select(carretPosition,carretPosition);
			
			// show the dialog box
			showDialogWithWidthRestriction(text, "Error!", 40, JOptionPane.ERROR_MESSAGE);
		}
		
		/**
		 * This method shows a file select dialog, and saves the current content in the console field of the GUI i text format to 
		 * the selected file.
		 */
		private void saveOutputMethod() {
			System.out.println();
			String fileName;
			saveFC = new JFileChooser(prefs.get(REGPATH_OUTPUTSAVE, "c:\\"));
			saveFC.setDialogTitle("Save console output");
			FileNameExtensionFilter filter = new FileNameExtensionFilter("Text file","txt");
			saveFC.setFileFilter(filter);
			saveFC.setSelectedFile(new File("c:\\consoleOutput.txt"));
			
			int returnVal = saveFC.showSaveDialog(mainWindow);
	
			if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = saveFC.getSelectedFile();
	            fileName = file.getAbsolutePath();
	            prefs.put(REGPATH_OUTPUTSAVE, fileName);

	        } else {
	        	fileName = "c:\\consoleOutput.txt";
	        }
			try {
				FileWriter outFile = new FileWriter(fileName);
				PrintWriter outWriter = new PrintWriter(outFile);

				outWriter.println(consoleTextArea.getText());
				outWriter.close();
				Frontend.getInstance().displayInfo("Console output saved");
			} catch (IOException e) {
				Frontend.getInstance().displayWarning("Error while saving output to file");
			}
			
		}
		
		/**
		 * This method displays a open file dialog and returns the path
		 * to the fil that the user selects
		 * 
		 * @param callingField
		 * @return
		 */
		private String openMethod(int callingField) {
		
			int returnVal;
			switch(callingField) {
				case(FIELD_MALWARE):
					returnVal = fc.showOpenDialog(this.samplePathField);
					break;
				case(FIELD_CONFIG):
					returnVal = fc.showOpenDialog(this.configField);
					break;
				case(FIELD_PROPS):
					returnVal = fc.showOpenDialog(this.propsField);
					break;
				default:
					returnVal = fc.showOpenDialog(this.samplePathField);
			}
			
			
			if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = fc.getSelectedFile();
	            return file.getAbsolutePath();

	        } else {
	        	return "";
	        }
		}
		
		/**
		 * Method that displays the 'About' text on the console
		 */
		private void aboutMethod(){
			this.clearMethod();
			consoleTextArea.setText(ABOUT_STRING);
		}
		
		/**
		 * Method that displays the help instruction text on the console
		 */
		private void helpMethod(){
			this.clearMethod();
			consoleTextArea.setText(HELP_STRING);
		}
		
		/**
		 * This method displays a close window warning and closes the main
		 * application if the user answers yes
		 */
		private void exitMethod(){
			final JOptionPane optionPane = new JOptionPane(
					EXIT_WARNING_QUESTION,
	                JOptionPane.QUESTION_MESSAGE,
	                JOptionPane.YES_NO_OPTION);
			
			final JDialog dialog = new JDialog(mainWindow, "Warning", true);
			dialog.setContentPane(optionPane);
			dialog.setDefaultCloseOperation( JDialog.DO_NOTHING_ON_CLOSE );
			dialog.setLocationRelativeTo(mainWindow);
			dialog.addWindowListener(this);
			
			optionPane.addPropertyChangeListener(
				    new PropertyChangeListener() {
				        public void propertyChange(PropertyChangeEvent e) {
				            String prop = e.getPropertyName();

				            if (dialog.isVisible() 
				             && (e.getSource() == optionPane)
				             && (prop.equals(JOptionPane.VALUE_PROPERTY))) {
				                dialog.setVisible(false);
				            }
				        }
				    });
				dialog.pack();
				dialog.setVisible(true);

				int value = ((Integer)optionPane.getValue()).intValue();
				if (value == JOptionPane.YES_OPTION) {
				    System.exit(0);
				} else if (value == JOptionPane.NO_OPTION) {
				    dialog.setVisible(false);
				}
		}
		
		/**
		 * This method is responsible for starting the GUI config file editor
		 * (opens in new frame)
		 */
		private void configEditMethod(){
			
			if(propsField.getText().equals(null) || propsField.getText().equals("")) {
				appendError(EMPTY_PROPS_PATH_ERROR);
			}
			else {
				try{
					PropertiesReader.initialize(propsField.getText());
				}catch(AlreadyInitializedException e){
					Frontend.getInstance().displayInfo("Attempted to initialize already initialized config reader.");
				}catch(IOException e){
					Frontend.getInstance().displayWarning("Unable to load properties file. Error message: "+e.getMessage());
				}finally {
					propsField.setEnabled(false);
					open3.setEnabled(false);
					clear3.setEnabled(false);
					GUIConfigEditor configEditor = new GUIConfigEditor(this);
				}
				
			}
		}
		
		
		/**
		 * This method is responsible for determining the calling object when an action event is raised.
		 * This happens when buttons or menu items in the GUI are invoked
		 */
		public void actionPerformed(ActionEvent e){

			if(e.getSource().equals(clear)){
				clearMethod();
			}
			else if(e.getSource().equals(scan)){
				scanMethod();
			}
			else if(e.getSource().equals(execute)) {
				executeMethod();
			}
			else if(e.getSource().equals(analyze)) {
				analyzeMethod();
			}
			else if(e.getSource().equals(menuItemExit)) {
				exitMethod();
			}
			else if(e.getSource().equals(open1) || e.getSource().equals(open2) || e.getSource().equals(open3)) {
				switch(Integer.parseInt(e.getActionCommand())){
					case(FIELD_MALWARE):
						samplePathField.setText(openMethod(FIELD_MALWARE));
						break;
					case(FIELD_CONFIG):
						configField.setText(openMethod(FIELD_CONFIG));
						break;
					case(FIELD_PROPS):
						propsField.setText(openMethod(FIELD_PROPS));
						break;
					default:
						samplePathField.setText(openMethod(FIELD_MALWARE));
						break;
				}
			}
			else if(e.getSource().equals(menuItemStart)) {
				initializePowerScan();
			}
			else if(e.getSource().equals(menuItemAbout)){
				aboutMethod();
			}
			else if(e.getSource().equals(menuItemHelp)){
				helpMethod();
			}
			else if(e.getSource().equals(clear1) || e.getSource().equals(clear2) || e.getSource().equals(clear3)) {
				switch(Integer.parseInt(e.getActionCommand())){
					case(FIELD_MALWARE):
						samplePathField.setText("");
						break;
					case(FIELD_CONFIG):
						configField.setText("");
						break;
					case(FIELD_PROPS):
						propsField.setText("");
						break;
					default:
						samplePathField.setText("");
						break;
				}
			}
			else if(e.getSource().equals(update)){
				updateMethod();
			}
			else if(e.getSource().equals(menuItemSave)) {
				saveOutputMethod();
			}
			else if(e.getSource().equals(menuItemConfEditor)){
				configEditMethod();
			}
		}

		/**
		 * Inherited method from the KeyListener, to take action as a key is typed
		 */
		public void keyTyped(KeyEvent arg0) {
			if(arg0.getKeyCode()==KeyEvent.VK_ENTER)
				initializePowerScan();
			else if(arg0.getKeyCode()==KeyEvent.VK_ESCAPE)
				exitMethod();
		}

		// More inherited methods from KeyListener
		/**
		 * (non-Javadoc)
		 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
		 */
		public void keyPressed(KeyEvent arg0) {
		}

		/**
		 * (non-Javadoc)
		 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
		 */
		public void keyReleased(KeyEvent arg0){}
		
		
		public static void main(String[] args) {
			GUI gui = new GUI();
		}

		/**
		 * Method implemented from the Runnable interface
		 * The GUI needs to run as its own thread to avoid freezing up
		 * when an operation is started. This blocks any new messages
		 * from being written to the console.
		 */
		@Override
		public void run() {
			consoleTextArea.setText("");

			try {
				switch(operation){
				case OPERATION_ANALYZE:
					appendInfo("Initiating dynamic analysis");
					disableOperations();
					if(samplePathField.getText().equals(null) || samplePathField.getText().equals("")) {
						appendError(EMPTY_SAMPLE_PATH_ERROR);
					}
					else {
						ArrayList<DynamicAnalysisResult> results = ps.executeInAnalysisTool( new File(samplePathField.getText()).getAbsolutePath() );
	
						if(results.size() > 0) {
							Frontend.getInstance().displayAnalysisToolsResult(results);
						}
					}
					setInDoneState();
					break;
				case OPERATION_SCAN:
					disableOperations();
					appendInfo("Initiating scan");
					if(samplePathField.getText().equals(null) || samplePathField.getText().equals("")) {
						appendError(EMPTY_SAMPLE_PATH_ERROR);
					}
					else {
						ArrayList<ScanResult> results = ps.scan(new File(samplePathField.getText()).getAbsolutePath());
						
						if(results.size() > 0) {
							Frontend.getInstance().displayScanResults(results);
						}
					}
					setInDoneState();
					break;
				case OPERATION_EXECUTE:
					disableOperations();
					appendInfo("Malware execution is requested");
					if(samplePathField.getText().equals(null) || samplePathField.getText().equals("")) {
						appendError(EMPTY_SAMPLE_PATH_ERROR);
					}
					else {
						ArrayList<ScanResult> results = ps.executeMalwareSample( new File(samplePathField.getText()).getAbsolutePath());
	
						if(results.size() > 0) {
							Frontend.getInstance().displayScanResults(results);
						}
					}
					setInDoneState();
					break;
				case OPERATION_INITIALIZE:
					disableOperations();
					appendInfo("System initializing");
					try {
						ps = new PowerScan(configField.getText(), propsField.getText());
						appendInfo("Now ready");
						setInAfterInitializationState();
					}
					catch(InvalidFileException e) {
						this.appendInfo("Invalid config files specified. Unable to load file(s)");
					}
					catch(ParserException e) {
						this.appendError("Unable to parse XML config file:\n"+e.getMessage());
					}
					catch(XMLValidationException e) {
						this.appendError("XML Validation failed:\n"+e.getMessage());
					}
					catch(FileNotFoundException e) {
						this.appendError("XML Validation failed:\n"+e.getMessage());	
					}
					finally {
						menuSystem.setEnabled(true);
						menuItemExit.setEnabled(true);
					}
					break;
				case OPERATION_UPDATE:
					disableOperations();	
					ArrayList<AVUpdateResult> results = ps.update();
					
					Frontend.getInstance().displayUpdateResults(results);
					
					setInDoneState();
	
				default:	
				}
			}
			catch(Exception e) {
				this.appendError("Unspecified error:" + e.getMessage());
				setInDoneState();
			}
	
		}

		/**
		 * Method implemented from the WindowListener. Shows information to the user
		 * if he tries to close the main frame. This is to avoid the application from
		 * being closed as an operation is executing. This will cause unpredicatble state
		 * in the system
		 */
		@Override
		public void windowActivated(WindowEvent arg0) {
			
		}

		/**
		 * Method implemented from the WindowListener. Shows information to the user
		 * if he tries to close the main frame. This is to avoid the application from
		 * being closed as an operation is executing. This will cause unpredicatble state
		 * in the system
		 */
		@Override
		public void windowClosed(WindowEvent arg0) {
			showDialogWithWidthRestriction(CLOSE_WARNING, "Warning", 40, JOptionPane.WARNING_MESSAGE);
			appendInfo(CLOSE_CONSOLE_WARNING);
			
		}
		
		/**
		 * Method implemented from the WindowListener. Shows information to the user
		 * if he tries to close the main frame. This is to avoid the application from
		 * being closed as an operation is executing. This will cause unpredicatble state
		 * in the system
		 */
		@Override
		public void windowClosing(WindowEvent arg0) {
			//JOptionPane.showMessageDialog(mainWindow, CLOSE_WARNING, "Warning!", JOptionPane.WARNING_MESSAGE);
			showDialogWithWidthRestriction(CLOSE_WARNING, "Warning", 40, JOptionPane.WARNING_MESSAGE);
			appendInfo(CLOSE_CONSOLE_WARNING);
			
		}

		/**
		 * (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
		 */
		@Override
		public void windowDeactivated(WindowEvent arg0) {
			
		}
		
		/**
		 * (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
		 */
		@Override
		public void windowDeiconified(WindowEvent arg0) {
			
		}

		/**
		 * (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
		 */
		@Override
		public void windowIconified(WindowEvent arg0) {
			
		}

		/**
		 * (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
		 */
		@Override
		public void windowOpened(WindowEvent arg0) {
			
		}
		
		/**
		 * This method is used to display a dialog box with a max width property
		 * @param msg The text to appear in the dialog box
		 * @param title The text to appear in the dialog box title field 
		 * @param width The max width of the dialog box (in characters)
		 * @param type The type of dialog box (using constants from JOptionPane, like WARNING_MESSAGE or ERROR_MESSAGE)
		 */
		private void showDialogWithWidthRestriction(String msg, String title, int width, int type ) {
			JOptionPane pane = getNarrowOptionPane(width);
			pane.setMessage(msg);
			pane.setMessageType(type);
			JDialog dialog = pane.createDialog(mainWindow, title);
			dialog.setVisible(true);
		}
		
		/**
		 * Helper method that overrides the JOptionPane class and creates a setter for the read-only value
		 * maxCharactersPerLineCount in JOptionPane
		 * @param maxCharactersPerLineCount
		 * @return a JOptionPane with the given width limitation
		 */
		private static JOptionPane getNarrowOptionPane(int maxCharactersPerLineCount) {
		   
		   // Inner class definition that overrides the JOptionPane class with 
		   class NarrowOptionPane extends JOptionPane {

			private static final long serialVersionUID = 1L;
			int maxCharactersPerLineCount;
			   
			   NarrowOptionPane(int maxCharactersPerLineCount) {
				   this.maxCharactersPerLineCount = maxCharactersPerLineCount;
			   }
			   
			   public int getMaxCharactersPerLineCount() {
				   return maxCharactersPerLineCount;
			   }
		   }

		   return new NarrowOptionPane(maxCharactersPerLineCount);
	   }
}
