/*
 * Created on 21. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.prefs.Preferences;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.simpleframework.xml.Serializer;
import org.simpleframework.xml.load.Persister;

import edu.ntnu.item.jt.configloader.XMLParser;
import edu.ntnu.item.jt.configloader.XMLValidator;
import edu.ntnu.item.jt.system.Frontend;
import edu.ntnu.item.jt.system.PowerScan;
import edu.ntnu.item.jt.system.configbeans.PowerScanBean;
import edu.ntnu.item.jt.system.configbeans.VMBean;
import edu.ntnu.item.jt.system.configbeans.VMwareHostBean;
import edu.ntnu.item.jt.system.errors.InvalidFileException;
import edu.ntnu.item.jt.system.errors.ParserException;
import edu.ntnu.item.jt.system.errors.XMLValidationException;

/**
 * This is the main class of the GUI config editor system, and represents the main
 * window of the editor. All events on this window is handled by private methods in this class. 
 * @author Thomas Langerud
 *
 */
public class GUIConfigEditor implements ActionListener, WindowListener, ChangeListener{
	
	private ArrayList<VMwareHostBean> hosts;
	private ArrayList<HostPanel> hostPanels;
	private JFrame configEditWindow;
	private JPanel configEditPanel;
	private JMenu menuFile, menuGo, menuDelete;
	private JMenuBar menuBar;
	private JMenuItem menuItemClose, menuItemSave, menuItemHostView, menuItemOpen, menuItemNew, menuItemDeleteHost, menuItemDeleteVM, menuItemDeleteAVE, menuItemDeleteTool;
	private JTabbedPane vmTabbedPane, hostTabbedPane;
	private JFileChooser saveFC, openFC;
	private PowerScanBean powerscan;
	private Preferences prefs;
	private GUI gui;
	
	
	/*
	 * Private constants
	 */
	final String EDIT_WINDOW_TITLE = "PowerScan - config editor";
	final String LOOK_AND_FEEL_PATH = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
	final String EXIT_WARNING_QUESTION = "Unsaved changes will be discarded, sure you want to close?";
	final String REGPATH_CONFIG_EDITOR = "edu.ntnu.item.jt.gui.confedit";
	
	/*
	 * Public constants
	 */
	public final String EDIT_AVE_BUTTON_COMMAND = "edit_ave";
	public final String NEW_AVE_BUTTON_COMMAND = "new_ave";
	public final String EDIT_DAT_BUTTON_COMMAND = "edit_dat";
	public final String NEW_DAT_BUTTON_COMMAND = "new_dat";
	public final String ADD_VM_BUTTON_COMMAND = "add_vm";
	public final String ADD_HOST_BUTTON_COMMAND = "add_host";
	
	
	/**
	 * Public constructor to create a new config editor window
	 * @param gui reference to the main GUI window.
	 */
	public GUIConfigEditor(GUI gui) {
		
		this.gui = gui;
		
		configEditWindow = new JFrame(EDIT_WINDOW_TITLE);
		
		configEditWindow.setLocation(100,200);
		configEditWindow.setSize(500,400);

		configEditWindow.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
				    
		try {
	        UIManager.setLookAndFeel(LOOK_AND_FEEL_PATH);
	    } 
		catch (InstantiationException e) {
			configEditWindow.setDefaultLookAndFeelDecorated(true);
	    } 
		catch (ClassNotFoundException e) {
			configEditWindow.setDefaultLookAndFeelDecorated(true);
	    } 
		catch (UnsupportedLookAndFeelException e) {
			configEditWindow.setDefaultLookAndFeelDecorated(true);
	    } 
		catch (IllegalAccessException e) {
			configEditWindow.setDefaultLookAndFeelDecorated(true);
	    }
		
		configEditWindow.setResizable(false);
		
		menuItemNew = new JMenuItem("New config");
		menuItemNew.setMnemonic(KeyEvent.VK_N);
		menuItemNew.addActionListener(this);
		menuItemSave = new JMenuItem("Save config ...");
		menuItemSave.setMnemonic(KeyEvent.VK_S);
		menuItemSave.addActionListener(this);
		menuItemSave.setEnabled(false);
		menuItemClose = new JMenuItem("Exit");
		menuItemClose.setMnemonic(KeyEvent.VK_X);
		menuItemClose.addActionListener(this);
		menuItemOpen = new JMenuItem("Open config ...");
		menuItemOpen.setMnemonic(KeyEvent.VK_O);
		menuItemOpen.addActionListener(this);
		
		menuItemHostView = new JMenuItem("Host/VM view");
		menuItemHostView.addActionListener(this);
		menuItemHostView.setMnemonic(KeyEvent.VK_H);
		menuItemHostView.setEnabled(false);
		
		menuItemDeleteHost = new JMenuItem("Delete active host");
		menuItemDeleteHost.addActionListener(this);
		menuItemDeleteHost.setMnemonic(KeyEvent.VK_H);
		menuItemDeleteHost.setEnabled(false);
		menuItemDeleteVM = new JMenuItem("Delete active VM");
		menuItemDeleteVM.addActionListener(this);
		menuItemDeleteVM.setMnemonic(KeyEvent.VK_V);
		menuItemDeleteVM.setEnabled(false);
		menuItemDeleteAVE = new JMenuItem("Delete active AVE");
		menuItemDeleteAVE.addActionListener(this);
		menuItemDeleteAVE.setMnemonic(KeyEvent.VK_E);
		menuItemDeleteAVE.setEnabled(false);
		menuItemDeleteTool = new JMenuItem("Delete active tool");
		menuItemDeleteTool.addActionListener(this);
		menuItemDeleteTool.setMnemonic(KeyEvent.VK_T);
		menuItemDeleteTool.setEnabled(false);
		
		menuFile = new JMenu("File");
		menuFile.setMnemonic(KeyEvent.VK_F);
		menuGo = new JMenu("View");
		menuGo.setMnemonic(KeyEvent.VK_G);
		menuDelete = new JMenu("Delete");
		menuDelete.setMnemonic(KeyEvent.VK_D);
		
		menuFile.add(menuItemNew);
		menuFile.add(menuItemOpen);
		menuFile.add(menuItemSave);
		menuFile.add(menuItemClose);
		menuGo.add(menuItemHostView);
		
		menuDelete.add(menuItemDeleteHost);
		menuDelete.add(menuItemDeleteVM);
		menuDelete.add(menuItemDeleteAVE);
		menuDelete.add(menuItemDeleteTool);
		
		menuBar = new JMenuBar(); 
		menuBar.add(menuFile);
		menuBar.add(menuGo);
		menuBar.add(menuDelete);
		
		configEditWindow.setJMenuBar(menuBar);
		configEditWindow.setVisible(true);
		
		hostPanels = new ArrayList<HostPanel>();
		hosts = new ArrayList<VMwareHostBean>();
		configEditPanel = new JPanel();
		
		// the registry reader must be initialized
		prefs = Preferences.userRoot();

	}

	private boolean openFile() {
		
		String fileName;
		openFC = new JFileChooser();
		openFC.setDialogTitle("Open config file");
		FileNameExtensionFilter filter = new FileNameExtensionFilter("XML file","xml");
		openFC.setFileFilter(filter);
		openFC.setSelectedFile(new File(prefs.get(REGPATH_CONFIG_EDITOR, "config\\config.xml")));
		
		int returnVal = openFC.showSaveDialog(configEditWindow);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = openFC.getSelectedFile();
            fileName = file.getAbsolutePath();

        } else {
        	fileName = "c:\\config.xml";
        }
		prefs.put(REGPATH_CONFIG_EDITOR, fileName);

		XMLParser parser = new XMLParser();
		try {
			powerscan = parser.parse(new File(fileName));
		}
		catch(InvalidFileException f) {
			if(Frontend.debug) f.printStackTrace();
			popupErrorMessage("Error parsing XML file. Please see log file for details");
		}
		catch(ParserException f) {
			if(Frontend.debug) f.printStackTrace();
			popupErrorMessage("Error parsing XML file. Please see log file for details");
		}
		
		createHostVMView();
		return true;
	}
    
	private void popupErrorMessage(String text) {
		
		JOptionPane.showMessageDialog(configEditWindow,
				text,
			    "Error",
			    JOptionPane.ERROR_MESSAGE);	
	}
	
	private void popupInfoMessage(String text) {
		
		JOptionPane.showMessageDialog(configEditWindow,
				text,
			    "Information",
			    JOptionPane.INFORMATION_MESSAGE);
	}
	private void newFile() {

		// remove the panels that might exist from an already open config file
		hostPanels = new ArrayList<HostPanel>();
		hosts = new ArrayList<VMwareHostBean>();
		configEditPanel = new JPanel();
		
		powerscan = new PowerScanBean();
		createNewHostInNewFile();
		createHostVMView();
	}
	
	/**
	 * Create the Host/VM view
	 */
	private void createHostVMView() {

		// disable menu items that should not be available in this view
		this.menuItemDeleteAVE.setEnabled(false);
		this.menuItemDeleteTool.setEnabled(false);
		this.menuItemHostView.setEnabled(false);
		this.menuItemSave.setEnabled(true);
		
		// make sure that the menu item that should be available are enabled
		this.menuItemDeleteHost.setEnabled(true);
		this.menuItemDeleteVM.setEnabled(true);
		
		configEditWindow.setVisible(false);
		configEditWindow.getContentPane().removeAll();
		
		configEditPanel = new JPanel();
		configEditPanel.setLayout(new GridLayout(2,1));
		
		hosts = powerscan.getVMWareHosts();
		
		hostTabbedPane = new JTabbedPane();
		vmTabbedPane = new JTabbedPane();
		
		hostTabbedPane.setTabPlacement(JTabbedPane.TOP);
		hostTabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

		hostPanels = new ArrayList<HostPanel>();
		
		int hostCounter = 1;
		
		for(VMwareHostBean host : hosts) {
			HostPanel hostPanel = new HostPanel(host,this);
			hostPanel.draw();
			hostTabbedPane.addTab("Host "+hostCounter++, hostPanel);
			hostPanels.add(hostPanel);											// adds all host panels to the host panel array
		}

		configEditPanel.add(hostTabbedPane);
		configEditPanel.add(vmTabbedPane);
		
		configEditWindow.getContentPane().add(configEditPanel);
		
		hostTabbedPane.addChangeListener(this);
		configEditWindow.setVisible(true);
		
		changeActiveHost(0);
	}

	/**
	 * This method changes the view of virtual machines on the bottom half of the main view to
	 * show the VMs the belong to a given host
	 * @param newActiveHost
	 */
	private void changeActiveHost(int newActiveHost) {

		vmTabbedPane.removeAll();
		ArrayList<VMPanel> vmPanels = hostPanels.get(newActiveHost).getVmPanels();
		
		int vmCounter = 1;
		for(VMPanel vmPanel : vmPanels) {
			vmTabbedPane.addTab("VM"+ vmCounter, vmPanel);
			vmCounter++;
		}

	}
	
	/**
	 * Create the "Edit AV engine" view
	 */
	private void createEditAVEview() {
		
		configEditWindow.setVisible(false);
		configEditPanel.removeAll();
		configEditPanel.setLayout(new GridLayout(1,1));	
		
		AVEnginePanel avePanel = hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).getAvePanel();
		configEditPanel.add(avePanel);
		
		configEditWindow.setVisible(true);
		
		// enable menu items
		this.menuItemDeleteAVE.setEnabled(true);
		this.menuItemHostView.setEnabled(true);
	}
	
	/**
	 * Create the "Edit DAT" view
	 */
	private void createEditDATview() {
		
		configEditWindow.setVisible(false);
		configEditPanel.removeAll();
		configEditPanel.setLayout(new GridLayout(1,1));
		
		DynamicToolPanel datPanel = hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).getDatPanel();
		configEditPanel.add(datPanel);
		configEditWindow.setVisible(true);
		
		//enable menu items
		this.menuItemDeleteTool.setEnabled(true);
		this.menuItemHostView.setEnabled(true);
	}
	
	/**
	 * Update the information in the beans from what is currently in the text fields, validate
	 * and then write the new XML structure to file.
	 */
	private void save() {
		
		ArrayList<VMwareHostBean> vmwareHosts = new ArrayList<VMwareHostBean>(); 
		
		for(HostPanel hostPanel : hostPanels) {
			hostPanel.save();
			vmwareHosts.add(hostPanel.getVmHostBean());
		}
		powerscan.setVMWareHosts(vmwareHosts);
		
		// Write to a temp file using the Simple XML framework
		// First, create the temp file
		Serializer serializer = new Persister();
		
		File tempResult = null;
		try {
			tempResult = File.createTempFile("tempXML", "PS", new File("config\\"));
			tempResult.deleteOnExit();
		}
		catch(IOException e) {
			return;
		}
		
		try {
			// write to the temp file
			serializer.write(powerscan, tempResult);
		}
		catch(Exception e) {
			return;
		}
		
		// validate temp file
		XMLValidator validator = new XMLValidator();
		try {
			validator.validateXSD(tempResult.getAbsolutePath());
			Frontend.getInstance().displayInfo("XML validation completed");
		}
		catch(XMLValidationException e) {
			Frontend.getInstance().displayWarning("XML validation failed: \n"+e.getMessage()); 
			return;
		}
		catch(FileNotFoundException e) {
			Frontend.getInstance().displayWarning("XML validation failed: \n"+e.getMessage()); 
			return;
		}
		
		// now save to the permanent file
		String fileName;
		saveFC = new JFileChooser();
		saveFC.setDialogTitle("Save config file");
		FileNameExtensionFilter filter = new FileNameExtensionFilter("XML file","xml");
		saveFC.setFileFilter(filter);
		saveFC.setSelectedFile(new File(prefs.get(REGPATH_CONFIG_EDITOR, "config\\config.xml")));
		
		int returnVal = saveFC.showSaveDialog(configEditWindow);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = saveFC.getSelectedFile();
            fileName = file.getAbsolutePath();
        } 
		else {
        	fileName = "c:\\config.xml";
        }
		
		prefs.put(REGPATH_CONFIG_EDITOR, fileName);
		
		// create the permanent file... 
		File result = new File(fileName);
		
		// ... and write to it
		try {
			serializer.write(powerscan, result);	
		}
		catch(Exception e) {
			popupErrorMessage("Error saving XML document.");
			return;
		}

		popupInfoMessage("Save completed");
	}
	
	/**
	 * Exit the config editor without asking the user for confirmation
	 */
	private void closeImmediateMethod() {
		configEditWindow.setVisible(false);
		configEditWindow.dispose();
	}
	
	/**
	 * Popup a dialog box asking the user for confirmation. If "Yes" is pressed, the config
	 * editor is exited.
	 */
	private void closeAskMethod() {
		final JOptionPane optionPane = new JOptionPane(
				EXIT_WARNING_QUESTION,
                JOptionPane.QUESTION_MESSAGE,
                JOptionPane.YES_NO_OPTION);
		
		final JDialog dialog = new JDialog(configEditWindow, "Warning", true);
		dialog.setContentPane(optionPane);
		dialog.setDefaultCloseOperation( JDialog.DO_NOTHING_ON_CLOSE );
		dialog.setLocationRelativeTo(configEditWindow);
		dialog.addWindowListener(this);
		
		optionPane.addPropertyChangeListener(
			    new PropertyChangeListener() {
			        public void propertyChange(PropertyChangeEvent e) {
			            String prop = e.getPropertyName();

			            if (dialog.isVisible() 
			             && (e.getSource() == optionPane)
			             && (prop.equals(JOptionPane.VALUE_PROPERTY))) {
			                dialog.setVisible(false);
			            }
			        }
			    });
			dialog.pack();
			dialog.setVisible(true);

			int value = ((Integer)optionPane.getValue()).intValue();
			if (value == JOptionPane.YES_OPTION) {
				closeImmediateMethod();
			} else if (value == JOptionPane.NO_OPTION) {
			    dialog.setVisible(false);
			}
	}
	
	/**
	 * Creates a new Anti-virus engine and associated panel
	 */
	private void createNewAVE() {
		hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).addAVEngine();
		createEditAVEview();
	}
	
	/**
	 * Creates a new dynamic analysis tool and associated panel
	 */
	private void createNewDAT() {
		hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).addDAT();
		createEditDATview();
	}
	
	/**
	 * Creates a new virtual machine and a new tab in the Host/VM view
	 */
	private void createNewVM() {
		hostPanels.get(hostTabbedPane.getSelectedIndex()).addVM();
		changeActiveHost(hostTabbedPane.getSelectedIndex());
	}

	/**
	 * Creates a new host, a new virtual machine connected to that host and new tabs for
	 * both the host and the virtual machine in the Host/VM view.
	 */
	private void createNewHost() {
		
		VMwareHostBean newHostBean = new VMwareHostBean();
		newHostBean.setHost("hostname");
		newHostBean.setPortnumber(902);
		newHostBean.setUsername("username");
		newHostBean.setPassword("password");
		
		ArrayList<VMBean> newVMBeanList = new ArrayList<VMBean>();
		VMBean newVMBean = new VMBean();
		newVMBean.setVmxPath("vmxPath");
		newVMBean.setUsername("username");
		newVMBean.setPassword("password");
		
		newVMBeanList.add(newVMBean);
		
		newHostBean.setVirtualMachines(newVMBeanList);

		HostPanel newHostPanel = new HostPanel(newHostBean, this);
		
		hostPanels.add(newHostPanel);
		hosts.add(newHostBean);
		
		createHostVMView();
	}
	
	/**
	 * Creates a new host and a new virtual machine.
	 * This method is used when creating a new config file.
	 */
	private void createNewHostInNewFile() {
		
		VMwareHostBean newHostBean = new VMwareHostBean();
		newHostBean.setHost("hostname");
		newHostBean.setUsername("username");
		newHostBean.setPassword("password");
		newHostBean.setPortnumber(902);
		
		ArrayList<VMBean> newVMBeanList = new ArrayList<VMBean>();
		VMBean newVMBean = new VMBean();
		newVMBean.setVmxPath("vmxPath");
		newVMBean.setUsername("username");
		newVMBean.setPassword("password");
		
		newVMBeanList.add(newVMBean);
		
		newHostBean.setVirtualMachines(newVMBeanList);

		HostPanel newHostPanel = new HostPanel(newHostBean, this);
		
		hostPanels.add(newHostPanel);
		hosts.add(newHostBean);
		
		powerscan.setVMWareHosts(hosts);
		
		createHostVMView();
	}
	
	/**
	 * Deletes the currently selected virtual machine
	 */
	private void deleteActiveVM() {
		
		hostPanels.get(hostTabbedPane.getSelectedIndex()).deleteVM(vmTabbedPane.getSelectedIndex());
		createHostVMView();
	}
	
	/**
	 * Deletes the currently selected host
	 */
	private void deleteActiveHost() {
		hostPanels.remove(hostTabbedPane.getSelectedIndex());
		hosts.remove(hostTabbedPane.getSelectedIndex());
		createHostVMView();
	}
	
	/**
	 * Deletes the currently viewed anti-virus engine
	 */
	private void deleteActiveAVE() {
		hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).deleteAVEngine();
		createHostVMView();
		
	}
	
	/**
	 * Deletes the currently selected dynamic analysis tool
	 */
	private void deleteActiveTool() {
		hostPanels.get(hostTabbedPane.getSelectedIndex()).getVmPanels().get(vmTabbedPane.getSelectedIndex()).deleteTool();
		createHostVMView();
	}
	// ActionListener methods
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource().equals(menuItemSave)) {
			save();
		}
		else if(e.getSource().equals(menuItemClose)) {
			closeAskMethod();
		}
		else if(e.getSource().equals(menuItemHostView)) {
			createHostVMView();
		}
		else if(e.getSource().equals(menuItemOpen)){
			openFile();
		}
		else if(e.getSource().equals(menuItemNew)) {
			newFile();
		}
		else if(e.getSource().equals(menuItemDeleteHost)) {
			deleteActiveHost();
		}
		else if(e.getSource().equals(menuItemDeleteVM)) {
			deleteActiveVM();
		}
		else if(e.getSource().equals(menuItemDeleteAVE)) {
			deleteActiveAVE();
		}
		else if(e.getSource().equals(menuItemDeleteTool)) {
			deleteActiveTool();
		}
		else if(e.getActionCommand().equals(EDIT_AVE_BUTTON_COMMAND)) {
			createEditAVEview();
		}
		else if(e.getActionCommand().equals(NEW_AVE_BUTTON_COMMAND)) {
			createNewAVE();
		}
		else if(e.getActionCommand().equals(EDIT_DAT_BUTTON_COMMAND)) {
			createEditDATview();
		}
		else if(e.getActionCommand().equals(NEW_DAT_BUTTON_COMMAND)) {
			createNewDAT();
		}
		else if(e.getActionCommand().equals(ADD_VM_BUTTON_COMMAND)){
			createNewVM();
		}
		else if(e.getActionCommand().equals(this.ADD_HOST_BUTTON_COMMAND)) {
			createNewHost();
		}
		else {
			
		}
		
	}
	// WindowListener methods
	@Override
	public void windowActivated(WindowEvent arg0) {	
	}
	@Override
	public void windowClosed(WindowEvent arg0) {	
	}
	@Override
	public void windowClosing(WindowEvent arg0) {	
	}
	@Override
	public void windowDeactivated(WindowEvent arg0) {	
	}
	@Override
	public void windowDeiconified(WindowEvent arg0) {
	}
	@Override
	public void windowIconified(WindowEvent arg0) {
	}
	@Override
	public void windowOpened(WindowEvent arg0) {	
	}
	
	// ChangeListener methods
	@Override
	public void stateChanged(ChangeEvent evt) {
		changeActiveHost( hostTabbedPane.getSelectedIndex() );
	}

}
