/*
 * Created on 3. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.ProgressMonitor;

import edu.ntnu.item.jt.configloader.PropertiesReader;
import edu.ntnu.item.jt.system.Frontend;

/**
 * Specialization of Frontend for use with GUI
 * 
 * @author Thomas Langerud, Jran Vagnby Lillesand
 *
 */
public class GUIFrontend extends Frontend implements PropertyChangeListener{

	private GUI gui;
	private static GUIFrontend instance;
	private ProgressMonitor progressMonitor;
	
	private GUIFrontend() {
		gui = new GUI();
	}
	
	/**
	 * Getter for the GUIFrontend singleton.
	 * 
	 * Returns a reference to the allowed instance of GUI Frontend.
	 * 
	 * @return reference to the GUI frontend.
	 */
	public synchronized static GUIFrontend getInstance(){
		if(instance==null) instance = new GUIFrontend();
		return instance;
	}
	
	/**
	 * Display information regarding the system.
	 * 
	 * @param message the message to be displayed.
	 */
	public void displayInfo(String message){
		gui.appendInfo(message);
	}
	
	/**
	 * Display warnings regarding the system.
	 * 
	 * @param message the message to be displayed.
	 */
	public void displayWarning(String message){
		gui.appendWarning(message);
	}

	/**
	 * Display errors regarding the system.
	 *  
	 * @param message the message to be displayed.
	 */
	public void displayError(String message){
		gui.appendError(message);
	}
	
	/**
	 * This method sleeps for a given amount of seconds, showing a progress dialog counting down. 
	 */
	@Override
	public void sleep(int secs) {

		try {
	        progressMonitor = new ProgressMonitor(null, "Sleeping for "+secs+" seconds", "", 0, secs);
	        progressMonitor.setProgress(0);
	        PropertyChangeSupport change = new PropertyChangeSupport(this);
	        change.addPropertyChangeListener(this);
	        
	        int i = 0;
	        int j = 0;
			while(i < secs && !progressMonitor.isCanceled()) {
				Thread.sleep(1000);
				i++;
				change.firePropertyChange("progress", j, i);
				j = i;
			}
		}
		catch(InterruptedException ie) {
			Frontend.getInstance().displayInfo("Sleep interrupted, please see log file");
		}
	}
	
    /**
     * Invoked when task's progress property changes.
     */
	@Override
	public void propertyChange(PropertyChangeEvent evt) {

        if ("progress" == evt.getPropertyName() ) {
            int progress = (Integer) evt.getNewValue();
            progressMonitor.setProgress(progress);
            String message = "Slept "+progress+" sec.\n";
            progressMonitor.setNote(message);
        }
	}
}
