/*
 * Created on 23. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.ntnu.item.jt.system.configbeans.VMBean;
import edu.ntnu.item.jt.system.configbeans.VMwareHostBean;

/**
 * This class is a representation of the VMwareHostBean as used in the config editor.
 * This panel is used in the "Host/VM" view in the editor.
 * @author Thomas Langerud
 *
 */
public class HostPanel extends JPanel{

	private VMwareHostBean vmHostBean;
	private GUIConfigEditor editor;
	private JLabel nameLabel, usernameLabel, passwordLabel, portLabel;
	private JTextField nameField, usernameField, passwordField, portField;
	private ArrayList<VMPanel> vmPanels;
	private JButton addHostButton, addVMButton;

	public HostPanel(VMwareHostBean vmHost, GUIConfigEditor editor) {
		super();

		vmHostBean = vmHost;
		this.editor = editor;
		vmPanels = new ArrayList<VMPanel>();

		for(VMBean vmBean : vmHostBean.getVirtualMachines() ) {
			VMPanel vmPanel = new VMPanel(vmBean, editor, this);
			vmPanel.draw();
			vmPanels.add(vmPanel);
		}

	}

	public VMwareHostBean getVmHostBean() {
		return vmHostBean;
	}

	public ArrayList<VMPanel> getVmPanels() {
		return vmPanels;
	}

	public void setVmPanels(ArrayList<VMPanel> vmPanels) {
		this.vmPanels = vmPanels;
	}

	public void setVmHostBean(VMwareHostBean vmHostBean) {
		this.vmHostBean = vmHostBean;
	}

	/**
	 * This method creates and populate the text panels and labels
	 * that belongs to this panel with data from the associated bean.
	 */
	public void draw() {
		
		this.setLayout(new GridLayout(5,2));
		nameLabel = new JLabel("Hostname");
		nameField = new JTextField(vmHostBean.getHost());
		
		portLabel = new JLabel("Port nr.");
		portField = new JTextField(""+vmHostBean.getPortnumber());
		
		usernameLabel = new JLabel("Username");
		usernameField = new JTextField(vmHostBean.getUsername());
		
		passwordLabel = new JLabel("Password");
		passwordField = new JTextField(vmHostBean.getPassword());
		
		this.add(nameLabel);
		this.add(nameField);
		this.add(portLabel);
		this.add(portField);
		this.add(usernameLabel);
		this.add(usernameField);
		this.add(passwordLabel);
		this.add(passwordField);

		
		addHostButton = new JButton("Add Host");
		addHostButton.setActionCommand(editor.ADD_HOST_BUTTON_COMMAND);
		addHostButton.addActionListener(editor);
		
		addVMButton = new JButton("Add Virtual Machine");
		addVMButton.setActionCommand(editor.ADD_VM_BUTTON_COMMAND);
		addVMButton.addActionListener(editor);
		
		this.add(addHostButton);
		this.add(addVMButton);
	}

	/**
	 * This methods fetches data from the text area on this panel and saves
	 * the data to the associated bean.
	 */
	public void save() {
		vmHostBean.setHost(nameField.getText());
		vmHostBean.setUsername(usernameField.getText());
		vmHostBean.setPassword(passwordField.getText());
		vmHostBean.setPortnumber( Integer.parseInt(portField.getText()) );
		
		ArrayList<VMBean> newVMBeans = new ArrayList<VMBean>();
		for(VMPanel vmPanel : vmPanels) {
			vmPanel.save();
			newVMBeans.add(vmPanel.getVmBean());
		}
		
		vmHostBean.setVirtualMachines(newVMBeans);	
	}

	/**
	 * This method updates the text fields on this panel with data from
	 * the associated bean.
	 */
	public void refresh() {
		nameField.setText(vmHostBean.getHost());
		usernameField.setText(vmHostBean.getUsername());
		passwordField.setText(vmHostBean.getPassword());
		portField.setText(""+vmHostBean.getPortnumber());
		
		for(VMPanel vmPanel : vmPanels) {
			vmPanel.refresh();
		}
		
	}
	
	/**
	 * This method adds a new virtual machine and the associated panel
	 * to the "Host/VM" view. 
	 */
	public void addVM() {
		
		VMBean newVMBean = new VMBean();
		newVMBean.setVmxPath("vmxPath");
		newVMBean.setUsername("username");
		newVMBean.setPassword("password");
		
		// Add a new panel
		VMPanel newVMPanel = new VMPanel(newVMBean, editor, this);
		newVMPanel.draw();
		vmPanels.add(newVMPanel);
		
		// Add a new bean
		synchronized(this) {
			ArrayList<VMBean> beans = vmHostBean.getVirtualMachines();
			beans.add(newVMBean);
			vmHostBean.setVirtualMachines(beans);
		}
		refresh();
	}
	
	/**
	 * This method deletes one virtual machine from the host that it belongs to.
	 * @param vmNumber The number of the VM within the host
	 */
	public void deleteVM(int vmNumber) {
		
		synchronized(this) {
			this.vmPanels.remove(vmNumber);
			ArrayList<VMBean> vmBeans = this.vmHostBean.getVirtualMachines();
			vmBeans.remove(vmNumber);
			vmHostBean.setVirtualMachines(vmBeans);
		}
			refresh();
		
	}
}
