/*
 * Created on 24. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import edu.ntnu.item.jt.system.configbeans.DynamicAnalysisToolBean;

/**
 * This class is a JPanel representation of the DynamicAnalysisToolBean, used in the
 * "Edit tool" view of the config editor. This panels is used in the tabbed pane 
 * view represented by the DynamicToolPanel class. 
 * @author Thomas Langerud
 *
 */
public class ToolPanel extends JPanel implements ActionListener{

	private DynamicAnalysisToolBean toolBean;
	public JTextField name, path, parameters, resultPath;
	public JComboBox executeMalwareBox;
	private GUIConfigEditor editor;
	private JButton saveChangesButton, addToolButton;
	private DynamicToolPanel parentPanel;
	
	public ToolPanel(DynamicAnalysisToolBean toolBean, GUIConfigEditor editor, DynamicToolPanel parentPanel) {
		super();
		this.editor = editor;
		this.toolBean = toolBean;
		this.parentPanel = parentPanel;

	}
	/**
	 * This method creates and populates the text fields and labels
	 * associated with this panel.
	 */
	public void draw() {
		this.setLayout(new GridLayout(6,2));
		
		this.add(new JLabel("Name"));
		name = new JTextField(toolBean.getName());
		this.add(name);
		
		this.add(new JLabel("Executable path"));
		path = new JTextField(toolBean.getToolExecutablePath());
		this.add(path);

		this.add(new JLabel("Parameters"));
		parameters = new JTextField(toolBean.getToolParameters());
		this.add(parameters);
		
		this.add(new JLabel("Result file path"));
		resultPath = new JTextField(toolBean.getToolResultFile());
		this.add(resultPath);
				
		String[] choices = new String[] {"Yes", "No"};
		executeMalwareBox = new JComboBox(choices);
		
		if(toolBean.getExecuteMalwareExplicitly()) {
			executeMalwareBox.setSelectedItem("Yes");
		}
		else executeMalwareBox.setSelectedItem("No");
		
		this.add(new JLabel("Execute sample explicitly?"));
		this.add(executeMalwareBox);
		
		saveChangesButton = new JButton("Save changes");
		saveChangesButton.addActionListener(this);
		saveChangesButton.setActionCommand("save");
		this.add(saveChangesButton);
		
		addToolButton = new JButton("Add tool");
		addToolButton.setActionCommand("add");
		addToolButton.addActionListener(parentPanel);
		this.add(addToolButton );
		
	}

	/**
	 * This method fetches the data from the text panels and stores it
	 * in the associated bean.
	 */
	public void save() {
		toolBean.setName(name.getText());
		toolBean.setToolExecutablePath(path.getText());
		toolBean.setToolParameters(parameters.getText());
		toolBean.setToolResultFile(resultPath.getText());
		
		if(executeMalwareBox.getSelectedItem().equals("Yes")) {
			toolBean.setExecuteMalwareExplicitly(true);
		}
		else toolBean.setExecuteMalwareExplicitly(false);
	}
	
	/**
	 * This method updates the information in the text panels from the data in the bean. 
	 */
	public void refresh() {
		name.setText(toolBean.getName());
		path.setText(toolBean.getToolExecutablePath());
		parameters.setText(toolBean.getToolParameters());
		resultPath.setText(toolBean.getToolResultFile());
		
		if(toolBean.getExecuteMalwareExplicitly()) {
			executeMalwareBox.setSelectedItem("Yes");
		}
		else executeMalwareBox.setSelectedItem("No");
	}

	public DynamicAnalysisToolBean getToolBean() {
		return toolBean;
	}

	public void setToolBean(DynamicAnalysisToolBean toolBean) {
		this.toolBean = toolBean;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("save")) {
			save();
			refresh();
		}
		
	}


	
	
}
