/*
 * Created on 23. april. 2008
 *
 */
package edu.ntnu.item.jt.GUI;

import java.awt.GridLayout;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.ntnu.item.jt.system.configbeans.AVEngineBean;
import edu.ntnu.item.jt.system.configbeans.DynamicAnalysisToolBean;
import edu.ntnu.item.jt.system.configbeans.VMBean;

/**
 * This class is a panel representation of the VMBean used in the config editor 
 * "Host/VM" view 
 * @author Thomas Langerud
 *
 */
public class VMPanel extends JPanel {
	
	private GUIConfigEditor editor;
	private JButton editAVEButton, addAVEButton, editDATButton, addDATButton;
	private VMBean vmBean;
	private AVEnginePanel avePanel;
	private DynamicToolPanel datPanel;
	private JTextField vmxPath, vmUsername, vmPassword;
	private HostPanel hostPanel;
	public boolean changeButtons = false;
	
	public VMPanel(VMBean vmBean, GUIConfigEditor editor, HostPanel hostPanel) {
		super();
		this.vmBean = vmBean;
		this.editor = editor;
		this.hostPanel = hostPanel;
		
		if(vmBean.getAvEngine() != null) {
			AVEnginePanel newAVEPanel = new AVEnginePanel( vmBean.getAvEngine(), editor);
			newAVEPanel.draw();
			this.avePanel = newAVEPanel;
		}
		else this.avePanel = null;
		
		if( (vmBean.getAnalysisTools() != null ) && ( vmBean.getAnalysisTools().size() > 0)) {
			DynamicToolPanel newDATPanel = new DynamicToolPanel( vmBean, editor);
			newDATPanel.draw();
			this.datPanel = newDATPanel;
		}
		else this.datPanel = null;
	}
	
	public VMBean getVmBean() {
		return vmBean;
	}


	public void setVmBean(VMBean vmBean) {
		this.vmBean = vmBean;
	}
	
	public AVEnginePanel getAvePanel() {
		return avePanel;
	}

	public void setAvePanel(AVEnginePanel avePanel) {
		this.avePanel = avePanel;
	}

	public DynamicToolPanel getDatPanel() {
		return datPanel;
	}

	public void setDatPanel(DynamicToolPanel datPanel) {
		this.datPanel = datPanel;
	}
	
	/**
	 * This method updates the panels on this panel with
	 * data from the associated bean. 
	 */
	public void refresh() {
		vmxPath.setText(vmBean.getVmxPath());	
		vmUsername.setText(vmBean.getUsername());
		vmPassword.setText(vmBean.getPassword());
		
		if(avePanel != null) {
			avePanel.refresh();
		}
		if(datPanel != null) {
			datPanel.refresh();
		}
		
		if(changeButtons) {
			if(addAVEButton != null) {
				this.remove(addAVEButton);
			}
			if(addDATButton != null) {
				this.remove(addDATButton);
			}
			
			if( vmBean.getAvEngine() != null && vmBean.getAnalysisTools() != null && ( vmBean.getAnalysisTools().size() != 0 )) {
				// VM has both AVE and tools
				
				this.add(editAVEButton);
				this.add(editDATButton);
			}
			else if( vmBean.getAvEngine() != null ){
				// VM has AVE

				this.add(editAVEButton);
				this.add(addDATButton);
				
			}
			else if( ( vmBean.getAnalysisTools() != null  ) && ( vmBean.getAnalysisTools().size() != 0 )) {
				// VM has tools

				this.add(addAVEButton );
				this.add(editDATButton);

			}
			else {
				// has no AVE and no tools

				this.add(addAVEButton );
				this.add(addDATButton);
			}			
			changeButtons = false;
		}	
	}
	
	/**
	 * This method creates and populates the text fields and labels that 
	 * belong to this panel. 
	 */
	public void draw() {
		
		JLabel tl4 = new JLabel("VMX path");
		vmxPath = new JTextField(vmBean.getVmxPath());
		
		JLabel tl5 = new JLabel("Username");
		vmUsername = new JTextField(vmBean.getUsername());
		
		JLabel tl6 = new JLabel("Password");
		vmPassword = new JTextField(vmBean.getPassword());
		
		this.setLayout(new GridLayout(4,2));
		
		this.add(tl4);
		this.add(vmxPath);
		
		this.add(tl5);
		this.add(vmUsername);
		
		this.add(tl6);
		this.add(vmPassword);
		
		editAVEButton = new JButton("Edit AV engine");
		editAVEButton.addActionListener(editor);
		editAVEButton.setActionCommand(editor.EDIT_AVE_BUTTON_COMMAND);
		
		editDATButton = new JButton("Edit analysis tools");
		editDATButton.addActionListener(editor);
		editDATButton.setActionCommand(editor.EDIT_DAT_BUTTON_COMMAND);
		
		addDATButton = new JButton("Add analysis tool");
		addDATButton.addActionListener(editor);
		addDATButton.setActionCommand(editor.NEW_DAT_BUTTON_COMMAND);
		
		addAVEButton = new JButton("Add AV engine");
		addAVEButton.addActionListener(editor);
		addAVEButton.setActionCommand(editor.NEW_AVE_BUTTON_COMMAND);
		
		if( vmBean.getAvEngine() != null && vmBean.getAnalysisTools() != null && ( vmBean.getAnalysisTools().size() != 0 )) {
			// VM has both AVE and tools
			this.add(editAVEButton);
			this.add(editDATButton);
		}
		else if( vmBean.getAvEngine() != null ){
			// VM has AVE
			this.add(editAVEButton);
			this.add(addDATButton);	
		}
		else if( ( vmBean.getAnalysisTools() != null  ) && ( vmBean.getAnalysisTools().size() != 0 )) {
			// VM has tools
			this.add(addAVEButton );
			this.add(editDATButton);
		}
		else {
			// has no AVE and no tools
			this.add(addAVEButton );
			this.add(addDATButton);
		}
	}
	
	/**
	 * This method fetches data from the text fields on this panel and 
	 * saves the data to the associated bean.
	 */
	public void save() {
		
		vmBean.setVmxPath(vmxPath.getText());
		vmBean.setUsername(vmUsername.getText());
		vmBean.setPassword(vmPassword.getText());
		if(avePanel != null) {
			avePanel.save();
			vmBean.setAvEngine(avePanel.getAvEngineBean());
		}
		if(datPanel != null) {
			datPanel.save();
			vmBean.setAnalysisTools(datPanel.getTools());
		}
	}
	
	/**
	 * This method adds a new AVEngine to a VM, and creates
	 * a new AVEnginePanel.
	 */
	public void addAVEngine() {
		
		hostPanel.save();
		
		AVEngineBean newAVEBean = new AVEngineBean();
		newAVEBean.setExecutablePath("path");
		newAVEBean.setName("name");
		newAVEBean.setParameters("parameters");
		newAVEBean.setRemoteLogFilePath("remote log file");
		newAVEBean.setAvUpdate(null);
		newAVEBean.setFilter(null);
		newAVEBean.setRealtimeScanBean(null);
		
		AVEnginePanel newPanel = new AVEnginePanel(newAVEBean, editor);
		avePanel = newPanel;
		avePanel.draw();
		
		vmBean.setAvEngine(newAVEBean);
		changeButtons = true;
		refresh();
	}
	
	/**
	 * This method adds a DynamicToolPanel
	 */
	public void addDAT() {
		
		hostPanel.save();
		
		DynamicAnalysisToolBean newDATBean = new DynamicAnalysisToolBean();
		newDATBean.setName("name");
		newDATBean.setToolExecutablePath("path");
		newDATBean.setToolParameters("parameters");
		newDATBean.setToolResultFile("resultpath");
		
		ArrayList<DynamicAnalysisToolBean> toolList = new ArrayList<DynamicAnalysisToolBean>();
		toolList.add(newDATBean);
		
		vmBean.setAnalysisTools(toolList);
		
		DynamicToolPanel newDATPanel = new DynamicToolPanel(vmBean, editor);
		
		datPanel = newDATPanel;
		datPanel.draw();
		
		changeButtons = true;
		refresh();
	}
	
	/**
	 * Method to delete the current AVEngine registred with a host
	 */
	public void deleteAVEngine() {
		this.avePanel = null;
		vmBean.setAvEngine(null);
		
		refresh();
	}
	/**
	 * Method to delete one tool registered with a host
	 */
	public void deleteTool() {
		
		if(datPanel != null) {
			if(datPanel.getToolPanels().size() > 1) {
				datPanel.deleteTool();
			}
			else {
				datPanel = null;
				vmBean.setAnalysisTools(null);
			}
		}
		
	}
}
