package edu.ntnu.item.jt.VMware;

import com.sun.jna.*;

/**
 * Interface to the functions defined in the vix.dll API. For more information about each of these functions, 
 * see the VIX API reference guide at http://pubs.vmware.com/vix-api/ReferenceGuide/.
 * 
 * None of the functions listed here are given any further documentation.
 * 
 * @author Jran Vagnby Lillesand
 */
public interface VixInterface extends Library{
	
	VixInterface INSTANCE = (VixInterface) Native.loadLibrary("vix", VixInterface.class);
	
    public int VixHost_Connect(int apiVersion,
	        int hostType,
	        String hostName,
	        int hostPort,
	        String userName,
	        String password,
	        int options,
	        int propertyListHandle,
	        Callback callbackProc,
	        byte[] clientData);

    public void VixHost_Disconnect(int hostHandle);
    public void Vix_ReleaseHandle(int handle);
    public long VixJob_Wait(int jobHandle, 
	    	int firstPropertyID, 
	    	int[] handle, 
	    	int propertyNone);
	    
    public int VixVM_Open(int hostHandle,
	        String vmxFilePathName,
	        Callback callbackProc,
	        byte[] clientData);

    public int VixVM_PowerOn(int vmHandle,
	        int powerOnOptions,
	        int propertyListHandle,
	        Callback callbackProc,
	        byte[] clientData);
	    
    public int VixVM_PowerOff(int vmHandle,
	        int powerOffOptions,
	        Callback callbackProc,
	        byte[] clientData);
	    
    public int VixVM_LoginInGuest(int vmHandle,
            String userName,
            String password,
            int options,
            Callback callbackProc,
            byte[] clientData);
	    
    public int VixVM_WaitForToolsInGuest(int vmHandle,
            int timeoutInSeconds,
            Callback callbackProc,
            byte[] clientData);
	    
    public int VixVM_RunProgramInGuest(int vmHandle, 
    		String guestProgramName, 
    		String commandLineArgs, 
    		int options,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
	    
    public int VixVM_CopyFileFromHostToGuest(int vmHandle,
            String hostPathName,
            String guestPathName,
            int options,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
    
    public int VixVM_CopyFileFromGuestToHost(int vmHandle,
            String guestPathName,
            String hostPathName,
            int options,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
	    
    public int VixVM_GetRootSnapshot(int vmHandle,
            int index, //always 0 when using VMware Server
            int[] snapshotHandle); //but what is this supposed to be? VixHandle &snapshotHandle
	    
    public int VixVM_RevertToSnapshot(int vmHandle,
            int snapshotHandle,
            int options,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
	    
    public int VixVM_CreateSnapshot(int vmHandle,
            String name,
            String description,
            int options,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
    
    /** This function is not supported for VMware Server in VIX 1.0 and 1.1
     * 
    public int VixVM_CreateDirectoryInGuest(int vmHandle,
            String pathName,
            int propertyListHandle,
            Callback callbackProc,
            byte[] clientData);
    */
    
    public int VixVM_GetNumRootSnapshots(int vmHandle,
    		int[] result);
    
    public String Vix_GetErrorText(long err,
            String locale);
}
