package edu.ntnu.item.jt.configloader;

import java.util.logging.Logger;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Error handler for the XML Validator.
 * 
 * @author Thomas Langerud
 *
 */
public class XMLErrorHandler implements org.xml.sax.ErrorHandler {

	private static Logger log = Logger.getLogger("edu.ntnu.item.jt.system");
	
	/**
	 * Private logger to assist in parsing log messages .
	 * 
	 * @param severity severity level of the occurence.
	 * @param message log message.
	 */
	private void log(int severity, String message){
		
		switch(severity){
		case LOG_INFO:
			log.info("XMLErrorHandler: " +message);
			break;
		case LOG_WARNING:
			log.warning("XMLErrorHandler: " +message);
			break;
		case LOG_CONFIG:
			log.config("XMLErrorHandler: " +message);
			break;
		case LOG_SEVERE:
			log.severe("XMLErrorHandler: " +message);
			break;
		case LOG_ENTERING:
			log.info("XMLErrorHandler: " +message);
			break;
		case LOG_EXITING:
			log.info("XMLErrorHandler: " +message);
			break;
		}
	}
	
	/*
	 * Log levels
	 */
	private static final int LOG_INFO = 0;
	private static final int LOG_WARNING = 1;
	private static final int LOG_CONFIG = 2;
	private static final int LOG_SEVERE = 3;
	private static final int LOG_ENTERING = 4;
	private static final int LOG_EXITING = 5;
	
	@Override
	public void error(SAXParseException exception) throws SAXException {
		log(LOG_WARNING, exception.getMessage());
		throw new SAXException(exception);
	}

	@Override
	public void fatalError(SAXParseException exception) throws SAXException {
		log(LOG_WARNING, exception.getMessage());
		throw new SAXException(exception);
	}

	@Override
	public void warning(SAXParseException exception) throws SAXException {
		log(LOG_WARNING, exception.getMessage());
		throw new SAXException(exception);
	}

}
