package edu.ntnu.item.jt.configloader;

import edu.ntnu.item.jt.VMware.VMwareServer;
import edu.ntnu.item.jt.system.Frontend;
import edu.ntnu.item.jt.system.PowerScan;
import edu.ntnu.item.jt.system.errors.*;
import edu.ntnu.item.jt.system.configbeans.*;
import org.simpleframework.xml.load.*;

import java.io.*;
import java.util.Date;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

/**
 * Parses an XML file to config beans.
 * 
 * @author Jran Vagnby Lillesand
 *
 */
public class XMLParser {

	private static Logger log = PowerScan.getLogger(PropertiesReader.getInstance().getString("log.system.scanner", "logs"+System.getProperty("file.separator")+"scanner.log"), "edu.ntnu.item.jt.system.scanner");
	
	/*
	 * Log levels
	 */
	private static final int LOG_INFO = 0;
	private static final int LOG_WARNING = 1;
	private static final int LOG_CONFIG = 2;
	private static final int LOG_SEVERE = 3;
	private static final int LOG_ENTERING = 4;
	private static final int LOG_EXITING = 5;
	
	/**
	 * Private logger to assist in parsing log messages .
	 * 
	 * @param severity severity level of the occurence.
	 * @param message log message.
	 */
	private void log(int severity, String message){
		
		switch(severity){
		case LOG_INFO:
			log.info("XMLParser: " +message);
			break;
		case LOG_WARNING:
			log.warning("XMLParser: " +message);
			break;
		case LOG_CONFIG:
			log.config("XMLParser: " +message);
			break;
		case LOG_SEVERE:
			log.severe("XMLParser: " +message);
			break;
		case LOG_ENTERING:
			log.info("XMLParser: " +message);
			break;
		case LOG_EXITING:
			log.info("XMLParser: " +message);
			break;
		}
	}
	
	/**
	 * Parses the supplied XML structured configuration file.
	 * 
	 * @param xmlConfig the XML file to be parsed
	 * @return the root bean containing references to every other piece of information in the configuration file.
	 * @throws InvalidFileException if the input file could not be read.
	 * @throws ParserException if an error occured while generating beans from the XML config file, such as malstructured XML. 
	 */
	public PowerScanBean parse(File xmlConfig) throws InvalidFileException, ParserException{
		log(LOG_WARNING,"Attempting to load data from "+xmlConfig);
		if(!xmlConfig.canRead()){
			log(LOG_WARNING,"Unable to read from XML file "+xmlConfig.getAbsolutePath());
			throw new InvalidFileException("Unable to load configuration file "+xmlConfig.getPath());
		}

		XMLValidator validator = new XMLValidator();
		
		log.info("Starting validation of the XML file");
		try {
			validator.validateXSD(xmlConfig.getAbsolutePath());
			log(LOG_INFO,"XML validation completed");
		}
		catch(XMLValidationException e) {
			log(LOG_SEVERE,"XML Validation failed. Message: "+e.getMessage());
			Frontend.getInstance().displayWarning("XML validation failed: \n"+e.getMessage()); 
		}
		catch(FileNotFoundException e) {
			log(LOG_SEVERE,"XML Validation failed. Message: "+e.getMessage());
			Frontend.getInstance().displayWarning("XML validation failed: \n"+e.getMessage()); 
		}
		
		log.info("Trying to read XML into beans.");
		
		PowerScanBean ac=null;
		try{
			ac = new Persister().read(PowerScanBean.class, xmlConfig);
		}catch(Exception e){
			log(LOG_SEVERE,"Deserialization of XML failed: "+e.getMessage());
			throw new ParserException(e.getMessage());
		}
		
		return ac;
	}	
}
