package edu.ntnu.item.jt.system;

import java.util.ArrayList;

/**
 * Container for information regarding the result of a antivirus engine update information.
 * 
 * @author Jran Vagnby Lillesand
 */
public class AVUpdateResult {
	
	/**
	 * Indicates that the update operation was not initiated.
	 */
	public static final int NOT_INITIATED = 0;
	
	/**
	 * Indicates that the update operation was successful.
	 */
	public static final int UPDATE_SUCCESSFUL = 1;
	
	/**
	 * Indicates that the given success indicator was not found in the log file.
	 */
	
	public static final int SUCCESS_INDICATOR_NOT_FOUND = 2;
	
	/**
	 * Indicates that no update log file was found.
	 */
	public static final int NO_LOG_FOUND = 3;
	
	/**
	 * Indicates that the whole log file has been returned for manual analysis.
	 */
	public static final int FULL_LOG_RETURNED = 4;
	
	/**
	 * Indicates that the XML states that the update operation does not report to log.
	 */
	public static final int LOG_NOT_SUPPORTED = 5;
	
	/**
	 * Indicates that the XML states that the engine does not support automatic update through PowerScan.
	 */
	public static final int UPDATE_NOT_SUPPORTED = 6;
	
	/**
	 * Indicates that the operation timed out.
	 */
	public static final int OPERATION_TIMEOUT = 7;
	
	private int resultState = NOT_INITIATED;

	private ArrayList<String> resultString;
	private Scanner scanner;
	
	
	
	public ArrayList<String> getResultString() {
		return resultString;
	}

	public void setResultString(ArrayList<String> resultString) {
		this.resultString = resultString;
	}

	/**
	 * Constructor for AVUpdateResult.
	 * 
	 * Protected as only internal classes should generate results.
	 * 
	 * @param scanner the scanner involved in the update operation.
	 */
	protected AVUpdateResult(Scanner scanner){
		this.scanner = scanner;
	}
	
	/**
	 * Get the name of the AV engine involved in this update operation.
	 * 
	 * @return name of the AV engine.
	 */
	public String getAVEngineName(){
		return scanner.getName();
	}

	/**
	 * Get the scanner object involved in the update operation.
	 * 
	 * @return scanner object involved in the update operation.
	 */
	public Scanner getScanner() {
		return scanner;
	}

	/**
	 * Get the result state of the update operation.
	 * 
	 * @return state of the update operation.
	 */
	public int getResultState() {
		return resultState;
	}

	/**
	 * Set he result state of the update operation
	 * 
	 * @param resultState state of the update operation.
	 */
	public void setResultState(int resultState) {
		this.resultState = resultState;
	}

}
