package edu.ntnu.item.jt.system;

import java.util.ArrayList;

import edu.ntnu.item.jt.VMware.*;

/**
 * Contains a generic representation of a virus scan result.
 * 
 * Is valid for one scan on one AVE.
 * 
 * @author Jran Vagnby Lillesand
 *
 */
public class ScanResult {
	
	private ArrayList<String> results;
	private Scanner scanner;
	//private GuestOS scanMachine;
	private long scanTime;
	
	public ScanResult(Scanner scanner){
		this.scanner = scanner;
		//this.scanMachine = scanMachine;
		
	}
	
	/**
	 * Returns a thorough representation of the scan result.
	 */
	public String toString(){
		String returnString = scanner.getName()+ " running from config " +scanner.getScanMachine().getvmxPath()+ " on host " +scanner.getScanMachine().getHost()+ ". Scan time: " +scanTime/1000+ " seconds. Reported result: \n";
		if(results == null || results.size() == 0){
			returnString += "No infection discovered";
		}else{
			for(String result: results){
				returnString += result +"\n";
			}
		}
		 		
		return returnString; 
	}
	
	/**
	 * Generates a summarized scan result on a relatively eye-pleasing format.
	 * 
	 * @return the scan result.
	 */
	public String resultSummary(){
		String returnString = scanner.getName()+ " running on " +scanner.getScanMachine().getHost()+ " reported:\n";
		
		if(results == null || results.size() == 0){
			returnString += "No infection discovered";
		}else{
			for(String result: results){
				returnString += result +"\n";
			}
		}
		
		return returnString;
	}
	
	/**
	 * Get the filtered scan results for the scan.
	 * 
	 * @return list of the scan results for this given scan.
	 */
	public ArrayList<String> getResults() {
		return results;
	}
	
	/**
	 * Set results for this scan. 
	 * 
	 * @param results list of matches returned from the scan.
	 */
	protected void setResults(ArrayList<String> results) {
		this.results = results;
	}
		
	/**
	 * Get the total time elapsed performing the scan.
	 * 
	 * @return scan time in milliseconds.
	 */
	public long getScanTime() {
		return scanTime;
	}
	
	/**
	 * Set a single results.
	 * 
	 * Typically used for setting short, simple results, such as error messages.
	 * 
	 * @param result
	 */
	public void setResult(String result){
		ArrayList<String> results = new ArrayList<String>();
		results.add(result);
		
		this.results = results;
	}
	
	/**
	 * Set the scan time for this operation.
	 * 
	 * @param scanTime time (in milliseconds) to perform scan.
	 */
	protected void setScanTime(long scanTime) {
		this.scanTime = scanTime;
	}
	
	/**
	 * Get the scanner used to perform this scan operation.
	 * 
	 * @return scanner used to perform scan.
	 */
	public Scanner getScanner(){
		return this.scanner;
	}
	
}
